/*
 Jonathan Berger
 Szymon Rusinkiewicz
 
 qsplat_gui_wx_main.cpp
 Where wxWindows starts

 Copyright (c) 1999-2000 The Board of Trustees of the
 Leland Stanford Junior University.  All Rights Reserved.
 */

#include "qsplat_gui_wx.h"

const char *QSPLAT_VERSION = "1.02";

class AppClass: public wxApp
{
public:
    bool OnInit(void);
    virtual int MainLoop();
    void DoEvent();
#ifdef DARWIN
    short MacHandleAEODoc(const WXEVENTREF event , WXEVENTREF reply);
#endif
};


// A macro that registers AppClass with wxWindows
IMPLEMENT_APP(AppClass)

bool AppClass::OnInit(void)
{
    theQSplatGUI = new QSplatWXGUI();
    theQSplatGUI->Go();
    return TRUE;
}

int AppClass::MainLoop()
{
    m_keepGoing = TRUE;

    while (m_keepGoing)
    {
        DoEvent();
    }
    return 0;
}
        
void AppClass::DoEvent()
{
    EventRecord event ;

    long sleepTime = 1;

    if (WaitNextEvent(everyEvent, &event, sleepTime, (RgnHandle) s_macCursorRgn))
    {
        MacHandleOneEvent( &event );
    }
    else
    {
        // idlers
        WindowPtr window = ::FrontWindow() ;
        if ( window )
            ::IdleControls( window ) ;

        wxTheApp->ProcessIdle() ;
    }
    if ( event.what != kHighLevelEvent )
        SetRectRgn( (RgnHandle) s_macCursorRgn , event.where.h ,
                    event.where.v ,  event.where.h + 1 , event.where.v + 1 ) ;
    if (!GUI->do_redraw)
    {
        bool done = GUI->idle();
        if (!GUI->do_redraw && done)
        {
            DeletePendingObjects() ;
            wxMacProcessNotifierAndPendingEvents() ;
            return;
        }
    }
    if (GUI->do_redraw)
    {
        GUI->redraw();
        GUI->do_redraw = false;
    }
    DeletePendingObjects() ;
    wxMacProcessNotifierAndPendingEvents() ;
}

#ifdef DARWIN
wxString wxMacFSSpec2MacFilename( const FSSpec *spec );

// Function from Frank Buss
// This allows one to double-click a .qs file and open QSplat on the Mac
short int AppClass::MacHandleAEODoc(const WXEVENTREF event , WXEVENTREF reply)
{
    // get desc list
    AEDescList docList;
    OSErr osError = AEGetParamDesc((AEDesc*)event, keyDirectObject, typeAEList, &docList);
    if (osError != noErr) return osError;
    // determine number of elements
    SInt32 numberOfItems;
    osError = AECountItems(&docList, &numberOfItems);
    if (osError != noErr) return osError;
    if (numberOfItems < 1) return noErr;
    // get first item
    DescType returnedType;
    FSSpec fileSpec;
    AEKeyword keyWord;
    Size actualSize;
    osError = AEGetNthPtr(&docList, 1, typeFSS, &keyWord, &returnedType, &fileSpec, sizeof(fileSpec), &actualSize);
    if (osError != noErr) return osError;
    // convert FSSpec to normal filename and open it
    //wxString firstFileToOpen = wxMacFSSpec2MacFilename(&fileSpec);
    const char *filename = wxMacFSSpec2MacFilename(&fileSpec);
    theQSplatGUI->OpenModel(filename);
    // close desc list
    AEDisposeDesc(&docList);
    return noErr;
}
#endif
